@@ -766,7 +766,7 @@ static long readtarheader(union hblock *hb, file_info *finfo, char *prefix)
   fchk=unoct(hb->dbuf.chksum, sizeof(hb->dbuf.chksum));
 
   DEBUG(5, ("checksum totals chk=%d fchk=%d chksum=%s\n",
-	    chk, fchk, hb->dbuf.size));
+	    chk, fchk, hb->dbuf.chksum));
 
   if (fchk != chk)
     {
@@ -780,17 +780,19 @@ static long readtarheader(union hblock *hb, file_info *finfo, char *prefix)
   unfixtarname(finfo->name + strlen(prefix), hb->dbuf.name,
 	       strlen(hb->dbuf.name) + 1);
 
-/* can't handle links at present */
-  if (hb->dbuf.linkflag != '0')
-    {
-      DEBUG(0, ("this tar file appears to contain some kind of link\n"));
+  /* can't handle links at present */
+  if (hb->dbuf.linkflag != '0') {
+    if (hb->dbuf.linkflag == 0) {
+      DEBUG(4, ("Warning: NULL link detected %s\n", finfo->name));
+    } else {
+      DEBUG(0, ("this tar file appears to contain some kind of link - ignoring\n"));
       return -2;
     }
+  }
 
-  if (unoct(hb->dbuf.mode, sizeof(hb->dbuf.mode)) & S_IFDIR)
-    {
-      finfo->mode=aDIR;
-    }
+  if ((unoct(hb->dbuf.mode, sizeof(hb->dbuf.mode)) & S_IFDIR)
+      || (*(finfo->name+strlen(finfo->name)-1) == '\\'))
+    finfo->mode=aDIR;
   else
     finfo->mode=0; /* we don't care about mode at the moment, we'll
 		    * just make it a regular file */
@@ -825,11 +827,12 @@ static long unoct(char *p, int ndgs)
   long value=0;
   /* Converts octal string to long, ignoring any non-digit */
 
-  while (--ndgs)
-    {
-      if (isdigit(*p))
-	value = (value << 3) | (long) (*p++ - '0');
-    }
+  while (--ndgs) {
+    if (isdigit(*p))
+      value = (value << 3) | (long) (*p - '0');
+    
+    p++;
+  }
 
   return value;
 }
